import argparse
import scholarmail.get_email as get_email
import scholarmail.gen_html as gen_html
import webbrowser
import os


def main():
    parser = argparse.ArgumentParser(
        description='a tool for convert google scholar mail to html.')
    parser.add_argument('--conf', type=str,
                        help='configure file', required=True)
    parser.add_argument('--html', type=str,
                        help='export html file', required=True, default='./')
    args = parser.parse_args()
    # path = os.path.abspath(os.path.dirname(__file__))

    name_csv, df = get_email.get_email(args.conf)
    if len(df):
        gen_html.gen_html(name_csv, args.conf, args.html, df)
        web_addr = 'file://' + \
            os.path.abspath(os.path.join(args.html, name_csv + '.html'))
        webbrowser.open(web_addr)

        print('Generated html file:')
        print(web_addr)
    else:
        print('No scholar emails')



if __name__ == '__main__':
    main()



#!/usr/bin/env python

import imaplib
import email
import socket
import socks
from lxml import etree
import pandas as pd
import time
import os
import json
from tqdm import tqdm
import re


def add_highlight(input_str, find_str_list):
    temp = input_str
    for fs in find_str_list:
        high_index = [(substr.start(),substr.end()) for substr in re.finditer(fs,input_str,re.IGNORECASE)]
        bias_len = len('<mark>' + '</mark>')
        for c, h in enumerate(high_index):
            b = c*bias_len
            temp = temp[:h[0]+b] + '<mark>' + temp[h[0]+b:h[1]+b] + '</mark>' + temp[h[1]+b:]
    return temp


def is_in_filter_list(string, filter_list=['Kerr']):
    filter_list = [f.lower() for f in filter_list]
    filter_string = string.lower().replace(',', ' ').replace(
        '-', ' ').replace(':', ' ').split(' ')
    set1 = set(filter_list)
    set2 = set(filter_string)
    return bool(set1.intersection(set2))


def get_email(path_conf):
    with open(path_conf, 'r') as load_f:
        conf = json.load(load_f)

    if conf['proxy']:
        print('setting proxy...')
        socks.set_default_proxy(
            socks.HTTP, conf['proxy_ip'], conf['proxy_port'])
        socket.socket = socks.socksocket

    conn = imaplib.IMAP4_SSL(conf['server'], conf['port'])
    print('server connected.')

    conn.login(conf['email'], conf['passkey'])
    print('login successful.')

    # auto select scholar based on address
    # do not need to label the emails as "scholar"
    conn.select('Inbox')
    _, d = conn.search(None, 'FROM', '"scholaralerts-noreply@google.com"')

    mail_list = d[0].split()
    print('get mails : ', len(mail_list))

    df = pd.DataFrame(columns=('title', 'link', 'info',
                      'abstract', 'source', 'filtered'))
    file_name = time.strftime("%Y%m%d%H%M%S", time.localtime())

    if len(mail_list) > 0:
        for l in tqdm(mail_list):
            _, data = conn.fetch(l, '(RFC822)')
            msg = email.message_from_string(data[0][1].decode('utf-8'))
            sub = msg.get('subject')
            mail_source = email.header.decode_header(sub)[0][0]
            if not mail_source.isascii():
                mail_source = mail_source.decode('utf-8')
            mail_content = msg.get_payload(decode=True).decode(
                'utf-8').replace('<br>', '#br#').replace('<b>', '').replace('</b>', '')

            selector = etree.HTML(mail_content)
            title = selector.xpath('/html/body/div/h3/a/text()')
            link = selector.xpath('/html/body/div/h3/a/@href')
            info = selector.xpath('/html/body/div/div/text()')
            info = [str(k).replace('\xa0', '') for k in info][:-1]
            tqdm.write('>>> solving mail: %s from %s get %d paper' % (l.decode('utf-8'),
                                                                    mail_source, len(title)))
            for n in range(len(title)):
                lk = link[n]

                try:
                    dic = {'title': title[n],
                        'link': lk,
                        'info': info[2*n],
                        'abstract': info[2*n+1],
                        'source': mail_source,
                        'filtered': is_in_filter_list(title[n], filter_list=conf['filter_list'])}

                    if conf['apply_highlight']:
                        dic['title'] = add_highlight(dic['title'], conf['highlight_list'])
                        dic['abstract'] = add_highlight(dic['abstract'], conf['highlight_list'])

                    df = df.append(dic, ignore_index=True)
                except:
                    tqdm.write('>>> error: '+title[n])

            # mark this email as deleted
            if conf['remove_mail']:
                conn.store(l, '+X-GM-LABELS', '\\Trash')

        temp = []
        for row in df['abstract']:
            temp.append(row.replace('#br#', '<br>'))
        df['abstract'] = temp

        df.sort_values('title', inplace=True, ignore_index=True)
        # df.to_csv(os.path.join(file_name + '.csv'), index=None)

        # move the deleted emails to trash
        if conf['remove_mail']:
            conn.expunge() 
        
        return file_name, df
    else:
        return file_name, df

    


if __name__ == '__main__':
    get_email()



#!/usr/bin/env python

import pandas as pd
from jinja2 import Environment, FileSystemLoader
import os
import json
import goslate
import concurrent.futures
import goslate
import socket
import socks


def get_translate(str_in, lang='zh-cn', max_workers=120):
    print('translating')
    socks.set_default_proxy(proxy_type=None, addr=None,port=None)
    socket.socket = socks.socksocket
    executor = concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
    gs = goslate.Goslate(executor=executor, service_urls=[
                         'https://translate.google.cn'])
    tr = gs.translate(str_in, target_language=lang)
    str_out = []
    for idx, k in enumerate(tr):
        str_out.append(str_in[idx]+'<br>'+k)
    return str_out


def concat_func(x):
    result = pd.Series({
        'link': x['link'][x['link'].index[0]],
        'info': ','.join(x['info'].unique()),
        'abstract': ','.join(x['abstract'].unique()),
        'source': '<br>'.join(x['source'].unique()),
        'source_count': len(x['source'])
    })
    return result


def gen_html(file_name, path_conf, path_html, df):
    # df = pd.read_csv(os.path.join('./csv', file_name + '.csv'))

    with open(path_conf, 'r') as load_f:
        conf = json.load(load_f)
    if conf['apply_filter']:
        df = df.drop(df[df['filtered'] == True].index)

    result = df.groupby(df['title']).apply(concat_func).reset_index()
    result.sort_values('source_count', inplace=True,
                       ignore_index=True, ascending=False)
    item_count = len(result)
    title_count = df['title'].value_counts().to_dict()
    source_count = df['source'].value_counts().to_dict()

    if conf['translate']:
        result['title'] = get_translate(result['title'].tolist())
        result['abstract'] = get_translate(result['abstract'].tolist())

    data = [k for k in result.T.to_dict().values()]

    loader = FileSystemLoader(os.path.dirname(os.path.abspath(__file__)))
    env = Environment(loader=loader)
    template = env.get_template('temperate.html')
    text = template.render(item_count=str(item_count),
                           source_count=source_count, data=data)

    f = open(os.path.join(path_html, file_name + '.html'), 'w')
    f.write(text)
    f.close()
    print('done')


if __name__ == '__main__':
    gen_html(input('input file name without extension:'))
